@@ -181,11 +181,11 @@ void File_Dsf::Header_Parse()
     Get_L8 (Size,                                               "Size");
 
     //Coherency check
-    if (File_Offset+Buffer_Offset+Size>File_Size)
+    if (Element_Offset<12 || File_Offset+Buffer_Offset+Size>File_Size)
     {
-        Size=File_Size-(File_Offset+Buffer_Offset);
         if (Element_Level<=2) //Incoherencies info only at the top level chunk
-            Fill(Stream_General, 0, "IsTruncated", "Yes");
+            IsTruncated(File_Offset+Buffer_Offset+Size, false, "DSF");
+        Size=File_Size-(File_Offset+Buffer_Offset);
     }
 
     Header_Fill_Code(Name, Ztring().From_CC4(Name));
