@@ -956,13 +956,20 @@ void File_Dts_Common::FileHeader_Parse()
     //https://www.atsc.org/wp-content/uploads/2015/03/Non-Real-Time-Content-Delivery.pdf
     if (IsSub || CC8(Buffer)!=CHUNK_DTSHDHDR || CC4(Buffer+8))
         return;
-    int64u StreamSize=-1;
-    int16u Bitw_Stream_Metadata;
-    bool Header_Parsed=false;
-    int64u Num_Samples_Orig_Audio_At_Max_Fs=0;
-    int32u Num_Frames_Total, TimeStamp, Max_Sample_Rate_Hz=0, Ext_Ss_Avg_Bit_Rate_Kbps=0, Ext_Ss_Peak_Bit_Rate_Kbps=0;
-    int16u Core_Ss_Bit_Rate_Kbps=0, Samples_Per_Frame_At_Max_Fs=0, Codec_Delay_At_Max_Fs=0;
-    int8u RefClockCode, TC_Frame_Rate=-1;
+    int64u StreamSize{ static_cast<int64u>(-1) };
+    int16u Bitw_Stream_Metadata{};
+    bool Header_Parsed{ false };
+    int64u Num_Samples_Orig_Audio_At_Max_Fs{ 0 };
+    int32u Num_Frames_Total{};
+    int32u TimeStamp{};
+    int32u Max_Sample_Rate_Hz{ 0 };
+    int32u Ext_Ss_Avg_Bit_Rate_Kbps{ 0 };
+    int32u Ext_Ss_Peak_Bit_Rate_Kbps{ 0 };
+    int16u Core_Ss_Bit_Rate_Kbps{ 0 };
+    int16u Samples_Per_Frame_At_Max_Fs{ 0 };
+    int16u Codec_Delay_At_Max_Fs{ 0 };
+    int8u RefClockCode{};
+    int8u TC_Frame_Rate{ static_cast<int8u>(-1) };
     while (StreamSize==-1 && Element_Size-Element_Offset>=16)
     {
         int64u Name, Size;
@@ -2311,6 +2318,7 @@ void File_Dts::Extensions2()
     {
         case 0x02000850:
         case 0xF14000D1:
+        case 0xF14000D4:
             Element_Name("X?");
             Presence.set(presence_Extended_X);
             break;
