@@ -6,7 +6,8 @@
 
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //
-// Source : http://www.wavpack.com/file_format.txt
+// Sources : http://www.wavpack.com/file_format.txt
+//           https://www.wavpack.com/WavPack5FileFormat.pdf
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
@@ -85,11 +86,21 @@ static const char* Wvpk_id(int8u ID)
         case 0x0B : return "correction file bitstream (wvc file)";
         case 0x0C : return "special extended bitstream for floating point data or integers > 24 bit";
         case 0x0D : return "contains channel count and channel_mask";
+        case 0x0E : return "contains compressed DSD audio"; // Ver 5.0+
+        // ids from here are "optional" so decoders should skip them if they don't understand them
         case 0x21 : return "RIFF header for .wav files (before audio)";
         case 0x22 : return "RIFF trailer for .wav files (after audio)";
         case 0x25 : return "some encoding details for info purposes";
         case 0x26 : return "16-byte MD5 sum of raw audio data";
         case 0x27 : return "non-standard sampling rate info";
+        // added with version 5.0 to handle non-wav files and block checksums
+        case 0x23 : return "header for non-wav files";
+        case 0x24 : return "trailer for non-wav files";
+        case 0x28 : return "target filename extension";
+        case 0x29 : return "16-byte MD5 sum of raw audio data with non-wav standard";
+        case 0x2A : return "new file configuration stuff";
+        case 0x2B : return "identities of non-MS channels";
+        case 0x2F : return "2- or 4-byte checksum of entire block";
         default:    return "";
     }
 }
@@ -154,7 +165,7 @@ void File_Wvpk::Streams_Finish()
         int64u BitDepth=dsf?1:(Wvpk_Resolution[(resolution1?1:0)*2+(resolution0?1:0)]);
         int64u Duration=Samples*1000/SamplingRate;
         int64u CompressedSize=File_Size-TagsSize;
-        int64u UncompressedSize=Duration*(num_channels?num_channels:(mono?1:2))*BitDepth*(SamplingRate<<(3*dsf))/8/1000;
+        int64u UncompressedSize=Duration*(num_channels?num_channels:(mono?1:2))*BitDepth*(static_cast<int64u>(SamplingRate)<<(3*dsf))/8/1000;
         float32 CompressionRatio=((float32)UncompressedSize)/CompressedSize;
         Fill(Stream_Audio, 0, Audio_StreamSize, CompressedSize, 3, true);
         Fill(Stream_Audio, 0, Audio_Duration, Duration, 10, true);
@@ -336,11 +347,12 @@ void File_Wvpk::Data_Parse()
         while (Element_Offset<Element_Size)
         {
             int32u total_samples=(int32u)-1, block_index=(int32u)-1, block_samples=0, flags, blocksize=(int32u)-1;
+            int8u block_index_u8=0, total_samples_u8=0;
             bool initial_block=true, final_block=true;
             if (!FromMKV)
             {
-                Skip_L1(                                            "track_no");
-                Skip_L1(                                            "index_no");
+                Get_L1 (block_index_u8,                             "block_index_u8");
+                Get_L1 (total_samples_u8,                           "total_samples_u8");
                 Get_L4 (total_samples,                              "total_samples");
                 Get_L4 (block_index,                                "block_index");
             }
@@ -351,10 +363,10 @@ void File_Wvpk::Data_Parse()
                 if (!FromMKV)
                 {
                     if (block_index==0) //Only the frame with block_index==0
-                        total_samples_FirstFrame=total_samples; //Note: total_samples is not trustable for a cutted file
+                        total_samples_FirstFrame=((int64u)total_samples_u8<<32)-total_samples_u8+total_samples; //Note: total_samples is not trustable for a cutted file
+                    block_index_LastFrame=((int64u)block_index_u8<<32)+block_index;
                     if (Frame_Count==1)
-                        block_index_FirstFrame=block_index; //Save the block_index of the first block
-                    block_index_LastFrame=block_index;
+                        block_index_FirstFrame=block_index_LastFrame; //Save the block_index of the first block
                     block_samples_LastFrame=block_samples;
                 }
                 Get_L4 (flags,                                      "flags");
@@ -386,10 +398,10 @@ void File_Wvpk::Data_Parse()
                     Skip_Flags(flags, 25,                           "sampling rate");
                     Skip_Flags(flags, 26,                           "sampling rate"); SamplingRate_Index=(int8u)(((flags>>23)&0xF)); Param_Info1(Wvpk_SamplingRate[SamplingRate_Index]);
                     Skip_Flags(flags, 27,                           "reserved");
-                    Skip_Flags(flags, 28,                           "reserved");
+                    Skip_Flags(flags, 28,                           "block contains checksum in last 2 or 4 bytes"); // Ver 5.0+
                     Skip_Flags(flags, 29,                           "use IIR for negative hybrid noise shaping");
                     Skip_Flags(flags, 30,                           "false stereo");
-                    Get_Flags (flags, 31, dsf,                      "dsf");
+                    Get_Flags (flags, 31, dsf,                      "dsf"); // Ver 5.0+
             }
             else
             {
@@ -443,7 +455,9 @@ void File_Wvpk::Data_Parse()
                     case 0x0D : id_0D(); break;
                     case 0x0E : id_0E(); break;
                     case 0x25 : id_25(); break;
+                    case 0x26 : id_26(); break;
                     case 0x27 : id_27(); break;
+                    case 0x29 : id_29(); break;
                     default   : if (word_size)
                                     Skip_XX(Size,                   "data");
                 }
@@ -576,7 +590,7 @@ void File_Wvpk::Data_Parse_Fill()
     {
         Fill(Stream_Audio, StreamPos_Last, Audio_SamplingRate, (Wvpk_SamplingRate[SamplingRate_Index]<<SamplingRate_Shift)<<(3*dsf));
         if (total_samples_FirstFrame!=(int32u)-1) //--> this is a valid value
-            Fill(Stream_Audio, 0, Audio_Duration, ((int64u)total_samples_FirstFrame)*1000/(Wvpk_SamplingRate[SamplingRate_Index]<<SamplingRate_Shift));
+            Fill(Stream_Audio, 0, Audio_Duration, ((int64u)total_samples_FirstFrame)*1000/((int64u)Wvpk_SamplingRate[SamplingRate_Index]<<SamplingRate_Shift));
     }
     Fill(Stream_Audio, 0, Audio_Encoded_Library_Settings, Encoded_Library_Settings);
     Fill(Stream_Audio, 0, Audio_BitRate_Mode, "VBR");
@@ -673,6 +687,21 @@ void File_Wvpk::id_0E()
     Skip_XX(Size-1,                                             "(Not parsed)");
 }
 
+//---------------------------------------------------------------------------
+void File_Wvpk::id_26()
+{
+    int128u MD5Stored;
+    Get_B16(MD5Stored,                                          "16-byte MD5 sum of raw audio data");
+
+    FILLING_BEGIN();
+    Ztring MD5_PerItem;
+    MD5_PerItem.From_UTF8(uint128toString(MD5Stored, 16));
+    while (MD5_PerItem.size() < 32)
+        MD5_PerItem.insert(MD5_PerItem.begin(), '0'); //Padding with 0, this must be a 32-byte string
+    Fill(Stream_Audio, 0, "MD5_Unencoded", MD5_PerItem);
+    FILLING_END();
+}
+
 //---------------------------------------------------------------------------
 void File_Wvpk::id_25()
 {
