@@ -362,6 +362,10 @@
 #if defined(MEDIAINFO_TGA_YES)
     #include "MediaInfo/Image/File_Tga.h"
 #endif
+#if defined(MEDIAINFO_WEBP_YES)
+    #include "MediaInfo/Image/File_WebP.h"
+#endif
+
 
 //---------------------------------------------------------------------------
 // Tag
@@ -395,6 +399,9 @@
 #if defined(MEDIAINFO_ISO9660_YES)
     #include "MediaInfo/Archive/File_Iso9660.h"
 #endif
+#if defined(MEDIAINFO_MACHO_YES)
+    #include "MediaInfo/Archive/File_MachO.h"
+#endif
 #if defined(MEDIAINFO_MZ_YES)
     #include "MediaInfo/Archive/File_Mz.h"
 #endif
@@ -586,6 +593,9 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_HEVC_YES)
         Parser.push_back(new File_Hevc());
     #endif
+    #if defined(MEDIAINFO_AVS3V_YES)
+        Parser.push_back(new File_Avs3V());
+#   endif
     #if defined(MEDIAINFO_AVSV_YES)
         Parser.push_back(new File_AvsV());
     #endif
@@ -605,7 +615,7 @@ File__MultipleParsing::File__MultipleParsing()
         Parser.push_back(new File_Vc1());
     #endif
     #if defined(MEDIAINFO_VC3_YES)
-        Parser.push_back(new File_Vc3());
+        //Parser.push_back(new File_Vc3());
     #endif
     #if defined(MEDIAINFO_Y4M_YES)
         Parser.push_back(new File_Y4m());
@@ -671,6 +681,9 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_FLAC_YES)
         Parser.push_back(new File_Flac());
     #endif
+    #if defined(MEDIAINFO_IAMF_YES)
+        Parser.push_back(new File_Iamf());
+    #endif
     #if defined(MEDIAINFO_IT_YES)
         Parser.push_back(new File_ImpulseTracker());
     #endif
@@ -781,6 +794,9 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_TIFF_YES)
         Parser.push_back(new File_Tiff());
     #endif
+    #if defined(MEDIAINFO_WEBP_YES)
+        Parser.push_back(new File_WebP());
+    #endif
 
     // Tag
     #if defined(MEDIAINFO_SPHERICALVIDEO_YES)
@@ -809,6 +825,9 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_ISO9660_YES)
         Parser.push_back(new File_Iso9660());
     #endif
+    #if defined(MEDIAINFO_MACHO_YES)
+        Parser.push_back(new File_MachO());
+    #endif
     #if defined(MEDIAINFO_MZ_YES)
         Parser.push_back(new File_Mz());
     #endif
@@ -849,6 +868,19 @@ File__MultipleParsing::~File__MultipleParsing()
         delete Parser[Pos]; //Parser[Pos]=NULL
 }
 
+//***************************************************************************
+// Streams management
+//***************************************************************************
+
+//---------------------------------------------------------------------------
+void File__MultipleParsing::Streams_Finish()
+{
+    if (Parser.size()==1 && Parser.front()->Status[IsAccepted])
+    {
+        Parser.front()->Finish();
+    }
+}
+
 //***************************************************************************
 // Buffer - Global
 //***************************************************************************
@@ -865,6 +897,7 @@ void File__MultipleParsing::Read_Buffer_Init()
         #else //MEDIAINFO_TRACE
             Parser[Pos]->Init(Config, Stream, Stream_More);
         #endif //MEDIAINFO_TRACE
+        Parser[Pos]->IsSub=IsSub;
         Parser[Pos]->File_Name=File_Name;
         Parser[Pos]->Open_Buffer_Init(File_Size);
     }
