@@ -16,6 +16,8 @@
 namespace MediaInfoLib
 {
 
+struct stts_struct;
+
 //***************************************************************************
 // Class File_Mpeg4_TimeCode
 //***************************************************************************
@@ -36,6 +38,9 @@ public :
     int64u  mdhd_Duration_TimeScale;
     int64u  tmcd_Duration;
     int64u  tmcd_Duration_TimeScale;
+    const std::vector<stts_struct>* DurationsPerFrame;
+    int64u  LastUsedOffset;
+    bool    AllFramesParsed;
 
     //Out
     int64s  Pos;
