@@ -247,6 +247,8 @@ struct complete_stream
         programs Programs; //Key is program_number
         std::vector<int16u> programs_List;
         size_t   Programs_NotParsedCount;
+        std::map<int8u, std::map<int32u, Ztring> > message_ids; //Key is message_id, langauge
+
 
         //Per IOD
         struct iod_es
@@ -785,6 +787,7 @@ private :
     void Descriptor_4D();
     void Descriptor_4E() {Skip_XX(Element_Size, "Data");};
     void Descriptor_4F() {Skip_XX(Element_Size, "Data");};
+    void NGA_component();
     void Descriptor_50();
     void Descriptor_51() {Skip_XX(Element_Size, "Data");};
     void Descriptor_52();
@@ -834,6 +837,7 @@ private :
     void Descriptor_7E() {Skip_XX(Element_Size, "Data");};
     void Descriptor_7F();
     void Descriptor_7F_06();
+    void Descriptor_7F_08();
     void Descriptor_7F_0F();
     void Descriptor_7F_15();
     void Descriptor_7F_19();
