@@ -1061,7 +1061,7 @@ void File_Riff::Header_Parse()
         else if (!File_Name.empty())
         {
             File F(File_Name);
-            F.GoTo(File_Offset+Buffer_Offset+8+Size);
+            F.GoTo(File_Offset+Buffer_Offset+8+Size_Complete); // TODO: remove this code and check non word aligned streams directly
             int8u Temp;
             if (F.Read(&Temp, 1))
             {
