@@ -28,6 +28,7 @@
 #endif //MEDIAINFO_EVENTS
 #include "tinyxml2.h"
 #include <cstring>
+#include <limits>
 using namespace tinyxml2;
 using namespace std;
 //---------------------------------------------------------------------------
@@ -209,37 +210,36 @@ void File_Ttml::Streams_Finish()
 {
     if (Time_End.IsSet() && Time_Begin.IsSet())
     {
-        int64s Duration=(Time_End-Time_Begin).ToMilliseconds();
-        Fill(Stream_General, 0, General_Duration, Duration);
-        Fill(Stream_Text, 0, Text_Duration, Duration);
-        if (!Time_Begin.IsTimed())
-            Fill(Stream_Text, 0, Text_TimeCode_FirstFrame, Time_Begin.ToString());
-        if (!Time_End.IsTimed() && Time_End>Time_Begin)
+        auto AdaptMediaTime=[&](TimeCode &TC)
         {
-            TimeCode LastFrame=Time_End;
-            --LastFrame;
-            Fill(Stream_Text, 0, Text_TimeCode_LastFrame, LastFrame.ToString());
-        }
-        auto MediaTimeToMilliseconds=[&](TimeCode TC)
-        {
-            auto Is1001fps=TC.Is1001fps();
-            if (TimeBase!=timeBase_media || (!Is1001fps && !TC.DropFrame())) // https://www.w3.org/TR/2018/REC-ttml2-20181108/#semantics-media-timing
-                return TC.ToMilliseconds();
+            if (Time_Begin.IsTimed() || TimeBase!=timeBase_media || (!TC.Is1001fps() && !TC.DropFrame())) // https://www.w3.org/TR/2018/REC-ttml2-20181108/#semantics-media-timing
+                return;
             auto Frames=TC.GetFrames();
             auto MS=((float)Frames)*1000/TC.GetFrameRate();
-            TC.Set1001fps(false);
-            TC.SetDropFrame(false);
-            TC.SetFrames(0);
-            auto Value=TC.ToMilliseconds();
+            auto Temp(TC);
+            Temp.Set1001fps(false);
+            Temp.SetDropFrame(false);
+            Temp.SetFrames(0);
+            auto Value=Temp.ToMilliseconds();
             Value+=float32_int64s(MS);
-            return Value;
+            TC.FromSeconds(Value / 1000.0);
         };
-        auto Time_Begin_ms = (int64s)MediaTimeToMilliseconds(Time_Begin);
-        auto Time_End_ms = (int64s)MediaTimeToMilliseconds(Time_End);
+        AdaptMediaTime(Time_Begin);
+        AdaptMediaTime(Time_End);
+        auto Duration = Time_End - Time_Begin;
+        auto Time_Begin_ms = (int64u)Time_Begin.ToMilliseconds();
+        auto Time_End_ms = (int64u)Time_End.ToMilliseconds();
+        auto Duration_ms = (int64u)(Time_End - Time_Begin).ToMilliseconds();
+        Fill(Stream_General, 0, General_Duration, Duration_ms);
+        Fill(Stream_Text, 0, Text_Duration, Duration_ms);
         Fill(Stream_Text, 0, Text_Duration_Start_Command, Time_Begin_ms);
         Fill(Stream_Text, 0, Text_Duration_Start, Time_Begin_ms);
         Fill(Stream_Text, 0, Text_Duration_End, Time_End_ms);
         Fill(Stream_Text, 0, Text_Duration_End_Command, Time_End_ms);
+        if (!Time_Begin.IsTimed())
+            Fill(Stream_Text, 0, Text_TimeCode_FirstFrame, Time_Begin.ToString());
+        if (!Time_End.IsTimed() && Time_End > Time_Begin)
+            Fill(Stream_Text, 0, Text_TimeCode_LastFrame, (Time_End - 1).ToString());
     }
     Fill(Stream_Text, 0, Text_FrameRate_Mode, "CFR");
     Fill(Stream_Text, 0, Text_Events_Total, FrameCount-EmptyCount);
@@ -316,6 +316,7 @@ void File_Ttml::Read_Buffer_Continue()
     // Root attributes
     bool IsSmpteTt=false, IsEbuTt=false, IsImsc1=false;
     int32u tickRate=0;
+    int32u TimeCodeFrameMax=0;
     const char* Tt_Attribute;
     Tt_Attribute=Root->Attribute("ittp:aspectRatio");
     if (!Tt_Attribute)
@@ -371,24 +372,6 @@ void File_Ttml::Read_Buffer_Continue()
         Fill(Stream_Text, 0, Text_FrameRate_Num, FrameRate_Int*FrameRateMultiplier_Num, 10, true);
         Fill(Stream_Text, 0, Text_FrameRate_Den, FrameRateMultiplier_Den, 10, true);
     }
-    if (!FrameRate_Int && !tickRate)
-    {
-        #if MEDIAINFO_ADVANCED
-            float64 FrameRate_F=Video_FrameRate_Rounded(Config->File_DefaultFrameRate_Get());
-            if (!FrameRate_F)
-                FrameRate_F=30; //"if no application defined frame rate applies, then thirty (30) frames per second" https://www.w3.org/TR/2018/REC-ttml2-20181108/#parameter-attribute-frameRate
-        #else //MEDIAINFO_ADVANCED
-            const float64 FrameRate_F=30;
-        #endif //MEDIAINFO_ADVANCED
-        FrameRate_Int=float64_int64s(FrameRate_F);
-        if (FrameRate_Int != FrameRate_F)
-        {
-            FrameRateMultiplier_Num=1000;
-            FrameRateMultiplier_Den=float64_int64s(FrameRate_Int/FrameRate_F*1000);
-            if (FrameRateMultiplier_Num==1000 && FrameRateMultiplier_Den==1001)
-                FrameRate_Is1001=true;
-        }
-    }
     Tt_Attribute=Root->Attribute("xml:lang");
     if (!Tt_Attribute)
         Tt_Attribute=Root->Attribute("lang");
@@ -429,6 +412,13 @@ void File_Ttml::Read_Buffer_Continue()
     {
         Fill(Stream_General, 0, General_Format_Profile, "SMPTE-TT");
         Fill(Stream_Text, 0, Text_Format_Profile, "SMPTE-TT");
+        if (!FrameRate_Int && tickRate >= 2000)
+        {
+            // Not clear but it seems that tickRate is actually frame rate x 1000 in several files without frame rate
+            auto Temp = float64_int64s(tickRate / 1000.0) - 1;
+            if (Temp <= numeric_limits<int32u>::max())
+                TimeCodeFrameMax = Temp;
+        }
     }
     if (IsEbuTt)
     {
@@ -440,6 +430,24 @@ void File_Ttml::Read_Buffer_Continue()
         Fill(Stream_General, 0, General_Format_Profile, "IMSC1");
         Fill(Stream_Text, 0, Text_Format_Profile, "IMSC1");
     }
+    if (!FrameRate_Int && !tickRate)
+    {
+        #if MEDIAINFO_ADVANCED
+            float64 FrameRate_F=Video_FrameRate_Rounded(Config->File_DefaultFrameRate_Get());
+            if (!FrameRate_F)
+                FrameRate_F=30; //"if no application defined frame rate applies, then thirty (30) frames per second" https://www.w3.org/TR/2018/REC-ttml2-20181108/#parameter-attribute-frameRate
+        #else //MEDIAINFO_ADVANCED
+            const float64 FrameRate_F=30;
+        #endif //MEDIAINFO_ADVANCED
+        FrameRate_Int=float64_int64s(FrameRate_F);
+        if (FrameRate_Int != FrameRate_F)
+        {
+            FrameRateMultiplier_Num=1000;
+            FrameRateMultiplier_Den=float64_int64s(FrameRate_Int/FrameRate_F*1000);
+            if (FrameRateMultiplier_Num==1000 && FrameRateMultiplier_Den==1001)
+                FrameRate_Is1001=true;
+        }
+    }
     string Rosetta_Profile, Rosetta_Version;
 
     tinyxml2::XMLElement*       div=NULL;
@@ -468,6 +476,8 @@ void File_Ttml::Read_Buffer_Continue()
                     {
                         if (!Time_Begin_New.FromString(Attribute))
                         {
+                            if (TimeCodeFrameMax && !Time_Begin_New.IsTimed())
+                                Time_Begin_New.SetFramesMax(TimeCodeFrameMax);
                             if (!Time_Begin.IsSet() || Time_Begin>Time_Begin_New)
                                 Time_Begin=Time_Begin_New;
                             if (!Time_End.IsSet() || Time_End<Time_Begin_New)
@@ -478,6 +488,8 @@ void File_Ttml::Read_Buffer_Continue()
                     {
                         if (!Time_End_New.FromString(Attribute))
                         {
+                            if (TimeCodeFrameMax && !Time_End_New.IsTimed())
+                                Time_End_New.SetFramesMax(TimeCodeFrameMax);
                             if (!Time_Begin.IsSet() || Time_Begin>Time_End_New)
                                 Time_Begin=Time_End_New;
                             if (!Time_End.IsSet() || Time_End<Time_End_New)
@@ -489,6 +501,8 @@ void File_Ttml::Read_Buffer_Continue()
                         TimeCode Time_Dur_New=Time_Template;
                         if (Time_Begin.IsSet() && !Time_Dur_New.FromString(Attribute))
                         {
+                            if (TimeCodeFrameMax && !Time_Dur_New.IsTimed())
+                                Time_Dur_New.SetFramesMax(TimeCodeFrameMax);
                             Time_End_New=Time_Begin_New;
                             Time_End_New+=Time_Dur_New;
                             if (!Time_Begin.IsSet() || Time_Begin>Time_End_New)
@@ -507,6 +521,8 @@ void File_Ttml::Read_Buffer_Continue()
                             {
                                 if (!Time_Begin_New.FromString(Attribute))
                                 {
+                                    if (TimeCodeFrameMax && !Time_Begin_New.IsTimed())
+                                        Time_Begin_New.SetFramesMax(TimeCodeFrameMax);
                                     if (!Time_Begin.IsSet() || Time_Begin>Time_Begin_New)
                                         Time_Begin=Time_Begin_New;
                                     if (!Time_End.IsSet() || Time_End<Time_Begin_New)
@@ -517,6 +533,8 @@ void File_Ttml::Read_Buffer_Continue()
                             {
                                 if (!Time_End_New.FromString(Attribute))
                                 {
+                                    if (TimeCodeFrameMax && !Time_End_New.IsTimed())
+                                        Time_End_New.SetFramesMax(TimeCodeFrameMax);
                                     if (!Time_Begin.IsSet() || Time_Begin>Time_End_New)
                                         Time_Begin=Time_End_New;
                                     if (!Time_End.IsSet() || Time_End<Time_End_New)
@@ -528,6 +546,8 @@ void File_Ttml::Read_Buffer_Continue()
                                 TimeCode Time_Dur_New=Time_Template;
                                 if (Time_Begin.IsSet() && !Time_Dur_New.FromString(Attribute))
                                 {
+                                    if (TimeCodeFrameMax && !Time_Dur_New.IsTimed())
+                                        Time_Dur_New.SetFramesMax(TimeCodeFrameMax);
                                     Time_End_New=Time_Begin_New;
                                     Time_End_New+=Time_Dur_New;
                                     if (!Time_Begin.IsSet() || Time_Begin>Time_End_New)
