@@ -69,6 +69,20 @@ const char* Av1_seq_profile(int8u seq_profile)
     }
 }
 
+//---------------------------------------------------------------------------
+const char* Av1_metadata_type(int8u metadata_type)
+{
+    switch (metadata_type)
+    {
+    case  0x1: return "METADATA_TYPE_HDR_CLL";
+    case  0x2: return "METADATA_TYPE_HDR_MDCV";
+    case  0x3: return "METADATA_TYPE_SCALABILITY";
+    case  0x4: return "METADATA_TYPE_ITUT_T35";
+    case  0x5: return "METADATA_TYPE_TIMECODE";
+    default: return "";
+    }
+}
+
 //---------------------------------------------------------------------------
 const char* Av1_frame_type[4] =
 {
@@ -141,17 +155,63 @@ void File_Av1::Streams_Fill()
 void File_Av1::Streams_Finish()
 {
     Fill(Stream_Video, 0, Video_Format_Settings_GOP, GOP_Detect(GOP));
-    if (!MasteringDisplay_ColorPrimaries.empty())
+
+    //Merge info about different HDR formats
+    auto HDR_Format = HDR.find(Video_HDR_Format);
+    if (HDR_Format != HDR.end())
     {
-        Fill(Stream_Video, 0, "HDR_Format", "SMPTE ST 2086");
-        Fill(Stream_Video, 0, "HDR_Format_Compatibility", "HDR10");
-        Fill(Stream_Video, 0, "MasteringDisplay_ColorPrimaries", MasteringDisplay_ColorPrimaries);
-        Fill(Stream_Video, 0, "MasteringDisplay_Luminance", MasteringDisplay_Luminance);
+        std::bitset<HdrFormat_Max> HDR_Present;
+        size_t HDR_FirstFormatPos = (size_t)-1;
+        for (size_t i = 0; i < HdrFormat_Max; ++i)
+            if (!HDR_Format->second[i].empty())
+            {
+                if (HDR_FirstFormatPos == (size_t)-1)
+                    HDR_FirstFormatPos = i;
+                HDR_Present[i] = true;
+            }
+        bool LegacyStreamDisplay = MediaInfoLib::Config.LegacyStreamDisplay_Get();
+        for (const auto& HDR_Item : HDR)
+        {
+            size_t i = HDR_FirstFormatPos;
+            size_t HDR_FirstFieldNonEmpty = (size_t)-1;
+            if (HDR_Item.first > Video_HDR_Format_Compatibility)
+            {
+                for (; i < HdrFormat_Max; ++i)
+                {
+                    if (!HDR_Present[i])
+                        continue;
+                    if (HDR_FirstFieldNonEmpty == (size_t)-1 && !HDR_Item.second[i].empty())
+                        HDR_FirstFieldNonEmpty = i;
+                    if (!HDR_Item.second[i].empty() && HDR_FirstFieldNonEmpty < HdrFormat_Max && HDR_Item.second[i] != HDR_Item.second[HDR_FirstFieldNonEmpty])
+                        break;
+                }
+            }
+            if (i == HdrFormat_Max && HDR_FirstFieldNonEmpty != (size_t)-1)
+                Fill(Stream_Video, 0, HDR_Item.first, HDR_Item.second[HDR_FirstFieldNonEmpty]);
+            else
+            {
+                ZtringList Value;
+                Value.Separator_Set(0, __T(" / "));
+                if (i != HdrFormat_Max)
+                    for (i = HDR_FirstFormatPos; i < HdrFormat_Max; ++i)
+                    {
+                        if (!LegacyStreamDisplay && HDR_FirstFormatPos != HdrFormat_SmpteSt2086 && i >= HdrFormat_SmpteSt2086)
+                            break;
+                        if (!HDR_Present[i])
+                            continue;
+                        Value.push_back(HDR_Item.second[i]);
+                    }
+                auto Value_Flat = Value.Read();
+                if (!Value.empty() && Value_Flat.size() > (Value.size() - 1) * 3)
+                    Fill(Stream_Video, 0, HDR_Item.first, Value.Read());
+            }
+        }
     }
+
     if (!maximum_content_light_level.empty())
-        Fill(Stream_Video, 0, "MaxCLL", maximum_content_light_level);
+        Fill(Stream_Video, 0, Video_MaxCLL, maximum_content_light_level);
     if (!maximum_frame_average_light_level.empty())
-        Fill(Stream_Video, 0, "MaxFALL", maximum_frame_average_light_level);
+        Fill(Stream_Video, 0, Video_MaxFALL, maximum_frame_average_light_level);
 }
 
 //***************************************************************************
@@ -231,7 +291,7 @@ void File_Av1::Header_Parse()
 void File_Av1::Data_Parse()
 {
     //Probing mode in case of raw stream //TODO: better reject of bad files
-    if (!IsSub && !Status[IsAccepted] && (!Element_Code || Element_Code>5))
+    if (!IsSub && !Status[IsAccepted] && (!Element_Code || Element_Code>6))
     {
         Reject();
         return;
@@ -245,6 +305,7 @@ void File_Av1::Data_Parse()
         case  0x3 : frame_header(); break;
         case  0x4 : tile_group(); break;
         case  0x5 : metadata(); break;
+        case  0x6 : frame(); break;
         case  0xF : padding(); break;
         default   : Skip_XX(Element_Size-Element_Offset,        "Data");
     }
@@ -470,11 +531,14 @@ void File_Av1::temporal_delimiter()
 //---------------------------------------------------------------------------
 void File_Av1::frame_header()
 {
+    /* bitstream conformance requires SeenFrameHeader is only 1 when OBU type is OBU_REDUNDANT_FRAME_HEADER
+     * since tile_group is not parsed, SeenFrameHeader cannot be relied upon as it is supposed to be reset by tile_group
     if (SeenFrameHeader)
     {
         Skip_XX(Element_Size,                                   "Duplicated data");
         return;
     }
+    */
     SeenFrameHeader=1;
 
     if (!sequence_header_Parsed)
@@ -489,6 +553,7 @@ void File_Av1::frame_header()
         int8u frame_type;
         TEST_SB_SKIP(                                           "show_existing_frame");
             BS_End();
+            SeenFrameHeader = 0;
             Skip_XX(Element_Size-Element_Offset,                "Data");
             return;
         TEST_SB_END();
@@ -516,7 +581,7 @@ void File_Av1::frame_header()
 //---------------------------------------------------------------------------
 void File_Av1::tile_group()
 {
-    Skip_XX(Element_Size,                                       "Data");
+    Skip_XX(Element_Size - Element_Offset,                      "Data");
 }
 
 //---------------------------------------------------------------------------
@@ -525,11 +590,13 @@ void File_Av1::metadata()
     //Parsing
     int64u metadata_type;
     Get_leb128 (metadata_type,                                  "metadata_type");
+    Param_Info1(Av1_metadata_type(metadata_type));
 
     switch (metadata_type)
     {
         case    1 : metadata_hdr_cll(); break;
         case    2 : metadata_hdr_mdcv(); break;
+        case    4 : metadata_itu_t_t35(); break;
         default   : Skip_XX(Element_Size-Element_Offset,        "Data");
     }
 }
@@ -545,7 +612,98 @@ void File_Av1::metadata_hdr_cll()
 void File_Av1::metadata_hdr_mdcv()
 {
     //Parsing
-    Get_MasteringDisplayColorVolume(MasteringDisplay_ColorPrimaries, MasteringDisplay_Luminance, true);
+    auto& HDR_Format = HDR[Video_HDR_Format][HdrFormat_SmpteSt2086];
+    if (HDR_Format.empty())
+    {
+        HDR_Format = __T("SMPTE ST 2086");
+        HDR[Video_HDR_Format_Compatibility][HdrFormat_SmpteSt2086] = "HDR10";
+    }
+    Get_MasteringDisplayColorVolume(HDR[Video_MasteringDisplay_ColorPrimaries][HdrFormat_SmpteSt2086], HDR[Video_MasteringDisplay_Luminance][HdrFormat_SmpteSt2086], true);
+}
+
+//---------------------------------------------------------------------------
+void File_Av1::metadata_itu_t_t35()
+{
+    //Parsing
+    int8u itu_t_t35_country_code;
+    Get_B1(itu_t_t35_country_code,                              "itu_t_t35_country_code");
+    if (itu_t_t35_country_code == 0xFF)
+        Skip_B1(                                                "itu_t_t35_country_code_extension_byte");
+
+    switch (itu_t_t35_country_code)
+    {
+    case 0xB5: metadata_itu_t_t35_B5(); break; // USA
+    }
+}
+
+//---------------------------------------------------------------------------
+void File_Av1::metadata_itu_t_t35_B5()
+{
+    int16u itu_t_t35_terminal_provider_code;
+    Get_B2(itu_t_t35_terminal_provider_code,                    "itu_t_t35_terminal_provider_code");
+
+    switch (itu_t_t35_terminal_provider_code)
+    {
+    case 0x003C: metadata_itu_t_t35_B5_003C(); break;
+    }
+}
+
+//---------------------------------------------------------------------------
+void File_Av1::metadata_itu_t_t35_B5_003C()
+{
+    int16u itu_t_t35_terminal_provider_oriented_code;
+    Get_B2(itu_t_t35_terminal_provider_oriented_code,           "itu_t_t35_terminal_provider_oriented_code");
+
+    switch (itu_t_t35_terminal_provider_oriented_code)
+    {
+    case 0x0001: metadata_itu_t_t35_B5_003C_0001(); break;
+    }
+}
+
+//---------------------------------------------------------------------------
+void File_Av1::metadata_itu_t_t35_B5_003C_0001()
+{
+    int8u application_identifier;
+    Get_B1(application_identifier,                              "application_identifier");
+
+    switch (application_identifier)
+    {
+    case 0x04: metadata_itu_t_t35_B5_003C_0001_04(); break;
+    }
+}
+
+//---------------------------------------------------------------------------
+void File_Av1::metadata_itu_t_t35_B5_003C_0001_04()
+{
+    Element_Info1("SMPTE ST 2094 App 4");
+
+    int8u application_version{};
+    bool IsHDRplus{ false }, tone_mapping_flag{};
+    Get_SMPTE_ST_2094_40(application_version, IsHDRplus, tone_mapping_flag);
+
+    FILLING_BEGIN();
+        auto& HDR_Format=HDR[Video_HDR_Format][HdrFormat_SmpteSt209440];
+        if (HDR_Format.empty())
+        {
+            HDR_Format=__T("SMPTE ST 2094 App 4");
+            HDR[Video_HDR_Format_Version][HdrFormat_SmpteSt209440].From_Number(application_version);
+            if (IsHDRplus)
+                HDR[Video_HDR_Format_Compatibility][HdrFormat_SmpteSt209440]=tone_mapping_flag?__T("HDR10+ Profile B"):__T("HDR10+ Profile A");
+        }
+    FILLING_END();
+}
+
+//---------------------------------------------------------------------------
+void File_Av1::frame()
+{
+    //Parsing
+    Element_Begin1("frame_header");
+    frame_header();
+    Element_End0();
+
+    Element_Begin1("tile_group");
+    tile_group();
+    Element_End0();
 }
 
 //---------------------------------------------------------------------------
